# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ...._types import Body, Query, Headers, NotGiven, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.whatsapp.phone_numbers import calling_setting_update_params
from ....types.whatsapp.phone_numbers.calling_setting_update_response import CallingSettingUpdateResponse
from ....types.whatsapp.phone_numbers.calling_setting_retrieve_response import CallingSettingRetrieveResponse

__all__ = ["CallingSettingsResource", "AsyncCallingSettingsResource"]


class CallingSettingsResource(SyncAPIResource):
    """Manage Whatsapp phone numbers"""

    @cached_property
    def with_raw_response(self) -> CallingSettingsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return CallingSettingsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CallingSettingsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return CallingSettingsResourceWithStreamingResponse(self)

    def retrieve(
        self,
        phone_number: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CallingSettingRetrieveResponse:
        """
        Get calling settings for a phone number

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not phone_number:
            raise ValueError(f"Expected a non-empty value for `phone_number` but received {phone_number!r}")
        return self._get(
            f"/v2/whatsapp/phone_numbers/{phone_number}/calling_settings",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CallingSettingRetrieveResponse,
        )

    def update(
        self,
        phone_number: str,
        *,
        enabled: bool,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CallingSettingUpdateResponse:
        """
        Enable or disable Whatsapp calling for a phone number

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not phone_number:
            raise ValueError(f"Expected a non-empty value for `phone_number` but received {phone_number!r}")
        return self._patch(
            f"/v2/whatsapp/phone_numbers/{phone_number}/calling_settings",
            body=maybe_transform({"enabled": enabled}, calling_setting_update_params.CallingSettingUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CallingSettingUpdateResponse,
        )


class AsyncCallingSettingsResource(AsyncAPIResource):
    """Manage Whatsapp phone numbers"""

    @cached_property
    def with_raw_response(self) -> AsyncCallingSettingsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncCallingSettingsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCallingSettingsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncCallingSettingsResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        phone_number: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CallingSettingRetrieveResponse:
        """
        Get calling settings for a phone number

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not phone_number:
            raise ValueError(f"Expected a non-empty value for `phone_number` but received {phone_number!r}")
        return await self._get(
            f"/v2/whatsapp/phone_numbers/{phone_number}/calling_settings",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CallingSettingRetrieveResponse,
        )

    async def update(
        self,
        phone_number: str,
        *,
        enabled: bool,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CallingSettingUpdateResponse:
        """
        Enable or disable Whatsapp calling for a phone number

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not phone_number:
            raise ValueError(f"Expected a non-empty value for `phone_number` but received {phone_number!r}")
        return await self._patch(
            f"/v2/whatsapp/phone_numbers/{phone_number}/calling_settings",
            body=await async_maybe_transform(
                {"enabled": enabled}, calling_setting_update_params.CallingSettingUpdateParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CallingSettingUpdateResponse,
        )


class CallingSettingsResourceWithRawResponse:
    def __init__(self, calling_settings: CallingSettingsResource) -> None:
        self._calling_settings = calling_settings

        self.retrieve = to_raw_response_wrapper(
            calling_settings.retrieve,
        )
        self.update = to_raw_response_wrapper(
            calling_settings.update,
        )


class AsyncCallingSettingsResourceWithRawResponse:
    def __init__(self, calling_settings: AsyncCallingSettingsResource) -> None:
        self._calling_settings = calling_settings

        self.retrieve = async_to_raw_response_wrapper(
            calling_settings.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            calling_settings.update,
        )


class CallingSettingsResourceWithStreamingResponse:
    def __init__(self, calling_settings: CallingSettingsResource) -> None:
        self._calling_settings = calling_settings

        self.retrieve = to_streamed_response_wrapper(
            calling_settings.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            calling_settings.update,
        )


class AsyncCallingSettingsResourceWithStreamingResponse:
    def __init__(self, calling_settings: AsyncCallingSettingsResource) -> None:
        self._calling_settings = calling_settings

        self.retrieve = async_to_streamed_response_wrapper(
            calling_settings.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            calling_settings.update,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["WhatsappMessageTemplateUpdateResponse", "Data", "DataWhatsappBusinessAccount"]


class DataWhatsappBusinessAccount(BaseModel):
    id: Optional[str] = None


class Data(BaseModel):
    id: Optional[str] = None

    category: Optional[Literal["MARKETING", "UTILITY", "AUTHENTICATION"]] = None

    components: Optional[List[object]] = None
    """Whatsapp template components (header, body, footer, buttons)"""

    created_at: Optional[datetime] = None

    language: Optional[str] = None

    name: Optional[str] = None

    record_type: Optional[str] = None

    rejection_reason: Optional[str] = None

    status: Optional[str] = None

    template_id: Optional[str] = None

    updated_at: Optional[datetime] = None

    whatsapp_business_account: Optional[DataWhatsappBusinessAccount] = None


class WhatsappMessageTemplateUpdateResponse(BaseModel):
    data: Optional[Data] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["TnUploadEntry"]


class TnUploadEntry(BaseModel):
    civic_address_id: Optional[str] = None
    """Identifies the civic address assigned to the phone number entry."""

    error_code: Optional[
        Literal[
            "internal_error",
            "unable_to_retrieve_default_location",
            "unknown_country_code",
            "unable_to_retrieve_location",
            "unable_to_retrieve_partner_info",
            "unable_to_match_geography_entry",
        ]
    ] = None
    """
    A code returned by Microsoft Teams if there is an error with the phone number
    entry upload.
    """

    error_message: Optional[str] = None
    """
    A message returned by Microsoft Teams if there is an error with the upload
    process.
    """

    internal_status: Optional[
        Literal[
            "pending_assignment",
            "in_progress",
            "all_internal_jobs_completed",
            "release_requested",
            "release_completed",
            "error",
        ]
    ] = None
    """Represents the status of the phone number entry upload on Telnyx."""

    location_id: Optional[str] = None
    """Identifies the location assigned to the phone number entry."""

    number_id: Optional[str] = None
    """Uniquely identifies the resource."""

    phone_number: Optional[str] = None
    """Phone number in E164 format."""

    status: Optional[Literal["pending_upload", "pending", "in_progress", "success", "error"]] = None
    """Represents the status of the phone number entry upload on Microsoft Teams."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ..._models import BaseModel

__all__ = ["VerifyMeta"]


class VerifyMeta(BaseModel):
    page_number: int

    page_size: int

    total_pages: int

    total_results: int



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from ...._models import BaseModel

__all__ = ["EmbeddingListEmbeddingModelsResponse", "Data"]


class Data(BaseModel):
    id: str
    """The model identifier"""

    created: int
    """Unix timestamp of when the model was created"""

    object: str
    """The object type, always 'model'"""

    owned_by: str
    """The organization that owns the model"""


class EmbeddingListEmbeddingModelsResponse(BaseModel):
    data: List[Data]
    """List of available embedding models"""

    object: str
    """The object type, always 'list'"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["IntegrationRetrieveResponse"]


class IntegrationRetrieveResponse(BaseModel):
    id: str

    available_tools: List[str]

    description: str

    display_name: str

    logo_url: str

    name: str

    status: Literal["disconnected", "connected"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from ..._models import BaseModel
from .profile_assignment_phone_numbers import ProfileAssignmentPhoneNumbers

__all__ = ["PhoneNumberAssignmentByProfileListPhoneNumberStatusResponse"]


class PhoneNumberAssignmentByProfileListPhoneNumberStatusResponse(BaseModel):
    records: List[ProfileAssignmentPhoneNumbers]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel

__all__ = ["SetiRetrieveBlackBoxTestResultsResponse", "Data", "DataBlackBoxTest"]


class DataBlackBoxTest(BaseModel):
    id: Optional[str] = None
    """The name of the black box test."""

    record_type: Optional[str] = None

    result: Optional[float] = None
    """The average result of the black box test over the last hour."""


class Data(BaseModel):
    black_box_tests: Optional[List[DataBlackBoxTest]] = None

    product: Optional[str] = None
    """The product associated with the black box test group."""

    record_type: Optional[str] = None


class SetiRetrieveBlackBoxTestResultsResponse(BaseModel):
    data: Optional[List[Data]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Required, TypedDict

from .._types import SequenceNotStr

__all__ = ["VerifyProfileCreateParams", "Call", "Flashcall", "Rcs", "SMS"]


class VerifyProfileCreateParams(TypedDict, total=False):
    name: Required[str]

    call: Call

    flashcall: Flashcall

    language: str

    rcs: Rcs

    sms: SMS

    webhook_failover_url: str

    webhook_url: str


class Call(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    app_name: str
    """
    The name that identifies the application requesting 2fa in the verification
    message.
    """

    code_length: int
    """The length of the verify code to generate."""

    default_verification_timeout_secs: int
    """
    For every request that is initiated via this Verify profile, this sets the
    number of seconds before a verification request code expires. Once the
    verification request expires, the user cannot use the code to verify their
    identity.
    """

    messaging_template_id: str
    """The message template identifier selected from /verify_profiles/templates"""

    whitelisted_destinations: SequenceNotStr[str]
    """Enabled country destinations to send verification codes.

    The elements in the list must be valid ISO 3166-1 alpha-2 country codes. If set
    to `["*"]`, all destinations will be allowed.
    """


class Flashcall(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    app_name: str
    """
    The name that identifies the application requesting 2fa in the verification
    message.
    """

    default_verification_timeout_secs: int
    """
    For every request that is initiated via this Verify profile, this sets the
    number of seconds before a verification request code expires. Once the
    verification request expires, the user cannot use the code to verify their
    identity.
    """

    whitelisted_destinations: SequenceNotStr[str]
    """Enabled country destinations to send verification codes.

    The elements in the list must be valid ISO 3166-1 alpha-2 country codes. If set
    to `["*"]`, all destinations will be allowed.
    """


class Rcs(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    app_name: str
    """
    The name that identifies the application requesting 2fa in the verification
    message.
    """

    code_length: int
    """The length of the verify code to generate."""

    default_verification_timeout_secs: int
    """
    For every request that is initiated via this Verify profile, this sets the
    number of seconds before a verification request code expires. Once the
    verification request expires, the user cannot use the code to verify their
    identity.
    """

    messaging_template_id: str
    """The message template identifier selected from /verify_profiles/templates"""

    sms_fallback: bool
    """Enable SMS fallback when RCS delivery fails."""

    whitelisted_destinations: SequenceNotStr[str]
    """Enabled country destinations to send verification codes.

    The elements in the list must be valid ISO 3166-1 alpha-2 country codes. If set
    to `["*"]`, all destinations will be allowed.
    """


class SMS(TypedDict, total=False, extra_items=object):  # type: ignore[call-arg]
    alpha_sender: Optional[str]
    """
    The alphanumeric sender ID to use when sending to destinations that require an
    alphanumeric sender ID.
    """

    app_name: str
    """
    The name that identifies the application requesting 2fa in the verification
    message.
    """

    code_length: int
    """The length of the verify code to generate."""

    default_verification_timeout_secs: int
    """
    For every request that is initiated via this Verify profile, this sets the
    number of seconds before a verification request code expires. Once the
    verification request expires, the user cannot use the code to verify their
    identity.
    """

    messaging_template_id: str
    """The message template identifier selected from /verify_profiles/templates"""

    whitelisted_destinations: SequenceNotStr[str]
    """Enabled country destinations to send verification codes.

    The elements in the list must be valid ISO 3166-1 alpha-2 country codes. If set
    to `["*"]`, all destinations will be allowed.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["UseCaseCategories"]

UseCaseCategories: TypeAlias = Literal[
    "2FA",
    "App Notifications",
    "Appointments",
    "Auctions",
    "Auto Repair Services",
    "Bank Transfers",
    "Billing",
    "Booking Confirmations",
    "Business Updates",
    "COVID-19 Alerts",
    "Career Training",
    "Chatbot",
    "Conversational / Alerts",
    "Courier Services & Deliveries",
    "Emergency Alerts",
    "Events & Planning",
    "Financial Services",
    "Fraud Alerts",
    "Fundraising",
    "General Marketing",
    "General School Updates",
    "HR / Staffing",
    "Healthcare Alerts",
    "Housing Community Updates",
    "Insurance Services",
    "Job Dispatch",
    "Legal Services",
    "Mixed",
    "Motivational Reminders",
    "Notary Notifications",
    "Order Notifications",
    "Political",
    "Public Works",
    "Real Estate Services",
    "Religious Services",
    "Repair and Diagnostics Alerts",
    "Rewards Program",
    "Surveys",
    "System Alerts",
    "Voting Reminders",
    "Waitlist Alerts",
    "Webinar Reminders",
    "Workshop Alerts",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import inbound_channel_update_params
from .._types import Body, Query, Headers, NotGiven, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.inbound_channel_list_response import InboundChannelListResponse
from ..types.inbound_channel_update_response import InboundChannelUpdateResponse

__all__ = ["InboundChannelsResource", "AsyncInboundChannelsResource"]


class InboundChannelsResource(SyncAPIResource):
    """Voice Channels"""

    @cached_property
    def with_raw_response(self) -> InboundChannelsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return InboundChannelsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> InboundChannelsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return InboundChannelsResourceWithStreamingResponse(self)

    def update(
        self,
        *,
        channels: int,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> InboundChannelUpdateResponse:
        """Update the number of Voice Channels for the US Zone.

        This allows your account to
        handle multiple simultaneous inbound calls to US numbers. Use this endpoint to
        increase or decrease your capacity based on expected call volume.

        Args:
          channels: The new number of concurrent channels for the account

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._patch(
            "/inbound_channels",
            body=maybe_transform({"channels": channels}, inbound_channel_update_params.InboundChannelUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=InboundChannelUpdateResponse,
        )

    def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> InboundChannelListResponse:
        """Returns the US Zone voice channels for your account.

        voice channels allows you
        to use Channel Billing for calls to your Telnyx phone numbers. Please check the
        <a href="https://support.telnyx.com/en/articles/8428806-global-channel-billing">Telnyx
        Support Articles</a> section for full information and examples of how to utilize
        Channel Billing.
        """
        return self._get(
            "/inbound_channels",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=InboundChannelListResponse,
        )


class AsyncInboundChannelsResource(AsyncAPIResource):
    """Voice Channels"""

    @cached_property
    def with_raw_response(self) -> AsyncInboundChannelsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncInboundChannelsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncInboundChannelsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncInboundChannelsResourceWithStreamingResponse(self)

    async def update(
        self,
        *,
        channels: int,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> InboundChannelUpdateResponse:
        """Update the number of Voice Channels for the US Zone.

        This allows your account to
        handle multiple simultaneous inbound calls to US numbers. Use this endpoint to
        increase or decrease your capacity based on expected call volume.

        Args:
          channels: The new number of concurrent channels for the account

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._patch(
            "/inbound_channels",
            body=await async_maybe_transform(
                {"channels": channels}, inbound_channel_update_params.InboundChannelUpdateParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=InboundChannelUpdateResponse,
        )

    async def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> InboundChannelListResponse:
        """Returns the US Zone voice channels for your account.

        voice channels allows you
        to use Channel Billing for calls to your Telnyx phone numbers. Please check the
        <a href="https://support.telnyx.com/en/articles/8428806-global-channel-billing">Telnyx
        Support Articles</a> section for full information and examples of how to utilize
        Channel Billing.
        """
        return await self._get(
            "/inbound_channels",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=InboundChannelListResponse,
        )


class InboundChannelsResourceWithRawResponse:
    def __init__(self, inbound_channels: InboundChannelsResource) -> None:
        self._inbound_channels = inbound_channels

        self.update = to_raw_response_wrapper(
            inbound_channels.update,
        )
        self.list = to_raw_response_wrapper(
            inbound_channels.list,
        )


class AsyncInboundChannelsResourceWithRawResponse:
    def __init__(self, inbound_channels: AsyncInboundChannelsResource) -> None:
        self._inbound_channels = inbound_channels

        self.update = async_to_raw_response_wrapper(
            inbound_channels.update,
        )
        self.list = async_to_raw_response_wrapper(
            inbound_channels.list,
        )


class InboundChannelsResourceWithStreamingResponse:
    def __init__(self, inbound_channels: InboundChannelsResource) -> None:
        self._inbound_channels = inbound_channels

        self.update = to_streamed_response_wrapper(
            inbound_channels.update,
        )
        self.list = to_streamed_response_wrapper(
            inbound_channels.list,
        )


class AsyncInboundChannelsResourceWithStreamingResponse:
    def __init__(self, inbound_channels: AsyncInboundChannelsResource) -> None:
        self._inbound_channels = inbound_channels

        self.update = async_to_streamed_response_wrapper(
            inbound_channels.update,
        )
        self.list = async_to_streamed_response_wrapper(
            inbound_channels.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.types import (
    TextToSpeechGenerateResponse,
    TextToSpeechListVoicesResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestTextToSpeech:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_generate(self, client: Telnyx) -> None:
        text_to_speech = client.text_to_speech.generate()
        assert_matches_type(TextToSpeechGenerateResponse, text_to_speech, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_generate_with_all_params(self, client: Telnyx) -> None:
        text_to_speech = client.text_to_speech.generate(
            aws={
                "language_code": "language_code",
                "lexicon_names": ["string"],
                "output_format": "output_format",
                "sample_rate": "sample_rate",
                "text_type": "text",
            },
            azure={
                "api_key": "api_key",
                "deployment_id": "deployment_id",
                "effect": "effect",
                "gender": "gender",
                "language_code": "language_code",
                "output_format": "output_format",
                "region": "region",
                "text_type": "text",
            },
            disable_cache=True,
            elevenlabs={
                "api_key": "api_key",
                "language_code": "language_code",
                "voice_settings": {"foo": "bar"},
            },
            inworld={"foo": "bar"},
            language="language",
            minimax={
                "language_boost": "language_boost",
                "pitch": 0,
                "response_format": "response_format",
                "speed": 0,
                "vol": 0,
            },
            output_type="binary_output",
            provider="aws",
            resemble={
                "api_key": "api_key",
                "format": "format",
                "precision": "precision",
                "sample_rate": "sample_rate",
            },
            rime={
                "response_format": "response_format",
                "sampling_rate": 0,
                "voice_speed": 0,
            },
            telnyx={
                "response_format": "response_format",
                "sampling_rate": 0,
                "temperature": 0,
                "voice_speed": 0,
            },
            text="text",
            text_type="text",
            voice="voice",
            voice_settings={"foo": "bar"},
        )
        assert_matches_type(TextToSpeechGenerateResponse, text_to_speech, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_generate(self, client: Telnyx) -> None:
        response = client.text_to_speech.with_raw_response.generate()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        text_to_speech = response.parse()
        assert_matches_type(TextToSpeechGenerateResponse, text_to_speech, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_generate(self, client: Telnyx) -> None:
        with client.text_to_speech.with_streaming_response.generate() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            text_to_speech = response.parse()
            assert_matches_type(TextToSpeechGenerateResponse, text_to_speech, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_voices(self, client: Telnyx) -> None:
        text_to_speech = client.text_to_speech.list_voices()
        assert_matches_type(TextToSpeechListVoicesResponse, text_to_speech, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_voices_with_all_params(self, client: Telnyx) -> None:
        text_to_speech = client.text_to_speech.list_voices(
            api_key="api_key",
            provider="aws",
        )
        assert_matches_type(TextToSpeechListVoicesResponse, text_to_speech, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_list_voices(self, client: Telnyx) -> None:
        response = client.text_to_speech.with_raw_response.list_voices()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        text_to_speech = response.parse()
        assert_matches_type(TextToSpeechListVoicesResponse, text_to_speech, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_list_voices(self, client: Telnyx) -> None:
        with client.text_to_speech.with_streaming_response.list_voices() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            text_to_speech = response.parse()
            assert_matches_type(TextToSpeechListVoicesResponse, text_to_speech, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncTextToSpeech:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_generate(self, async_client: AsyncTelnyx) -> None:
        text_to_speech = await async_client.text_to_speech.generate()
        assert_matches_type(TextToSpeechGenerateResponse, text_to_speech, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_generate_with_all_params(self, async_client: AsyncTelnyx) -> None:
        text_to_speech = await async_client.text_to_speech.generate(
            aws={
                "language_code": "language_code",
                "lexicon_names": ["string"],
                "output_format": "output_format",
                "sample_rate": "sample_rate",
                "text_type": "text",
            },
            azure={
                "api_key": "api_key",
                "deployment_id": "deployment_id",
                "effect": "effect",
                "gender": "gender",
                "language_code": "language_code",
                "output_format": "output_format",
                "region": "region",
                "text_type": "text",
            },
            disable_cache=True,
            elevenlabs={
                "api_key": "api_key",
                "language_code": "language_code",
                "voice_settings": {"foo": "bar"},
            },
            inworld={"foo": "bar"},
            language="language",
            minimax={
                "language_boost": "language_boost",
                "pitch": 0,
                "response_format": "response_format",
                "speed": 0,
                "vol": 0,
            },
            output_type="binary_output",
            provider="aws",
            resemble={
                "api_key": "api_key",
                "format": "format",
                "precision": "precision",
                "sample_rate": "sample_rate",
            },
            rime={
                "response_format": "response_format",
                "sampling_rate": 0,
                "voice_speed": 0,
            },
            telnyx={
                "response_format": "response_format",
                "sampling_rate": 0,
                "temperature": 0,
                "voice_speed": 0,
            },
            text="text",
            text_type="text",
            voice="voice",
            voice_settings={"foo": "bar"},
        )
        assert_matches_type(TextToSpeechGenerateResponse, text_to_speech, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_generate(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.text_to_speech.with_raw_response.generate()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        text_to_speech = await response.parse()
        assert_matches_type(TextToSpeechGenerateResponse, text_to_speech, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_generate(self, async_client: AsyncTelnyx) -> None:
        async with async_client.text_to_speech.with_streaming_response.generate() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            text_to_speech = await response.parse()
            assert_matches_type(TextToSpeechGenerateResponse, text_to_speech, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_voices(self, async_client: AsyncTelnyx) -> None:
        text_to_speech = await async_client.text_to_speech.list_voices()
        assert_matches_type(TextToSpeechListVoicesResponse, text_to_speech, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_voices_with_all_params(self, async_client: AsyncTelnyx) -> None:
        text_to_speech = await async_client.text_to_speech.list_voices(
            api_key="api_key",
            provider="aws",
        )
        assert_matches_type(TextToSpeechListVoicesResponse, text_to_speech, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_list_voices(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.text_to_speech.with_raw_response.list_voices()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        text_to_speech = await response.parse()
        assert_matches_type(TextToSpeechListVoicesResponse, text_to_speech, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_list_voices(self, async_client: AsyncTelnyx) -> None:
        async with async_client.text_to_speech.with_streaming_response.list_voices() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            text_to_speech = await response.parse()
            assert_matches_type(TextToSpeechListVoicesResponse, text_to_speech, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["PhoneNumberCampaignUpdateParams"]


class PhoneNumberCampaignUpdateParams(TypedDict, total=False):
    campaign_id: Required[Annotated[str, PropertyInfo(alias="campaignId")]]
    """The ID of the campaign you want to link to the specified phone number."""

    phone_number: Required[Annotated[str, PropertyInfo(alias="phoneNumber")]]
    """The phone number you want to link to a specified campaign."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel

__all__ = ["WhatsappContact", "Address", "Email", "Org", "Phone", "URL"]


class Address(BaseModel):
    city: Optional[str] = None

    country: Optional[str] = None

    country_code: Optional[str] = None

    state: Optional[str] = None

    street: Optional[str] = None

    type: Optional[str] = None

    zip: Optional[str] = None


class Email(BaseModel):
    email: Optional[str] = None

    type: Optional[str] = None


class Org(BaseModel):
    company: Optional[str] = None

    department: Optional[str] = None

    title: Optional[str] = None


class Phone(BaseModel):
    phone: Optional[str] = None

    type: Optional[str] = None

    wa_id: Optional[str] = None


class URL(BaseModel):
    type: Optional[str] = None

    url: Optional[str] = None


class WhatsappContact(BaseModel):
    addresses: Optional[List[Address]] = None

    birthday: Optional[str] = None

    emails: Optional[List[Email]] = None

    name: Optional[str] = None

    org: Optional[Org] = None

    phones: Optional[List[Phone]] = None

    urls: Optional[List[URL]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import porting_phone_number_list_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from .._base_client import AsyncPaginator, make_request_options
from ..types.porting_phone_number_list_response import PortingPhoneNumberListResponse

__all__ = ["PortingPhoneNumbersResource", "AsyncPortingPhoneNumbersResource"]


class PortingPhoneNumbersResource(SyncAPIResource):
    """Endpoints related to porting orders management."""

    @cached_property
    def with_raw_response(self) -> PortingPhoneNumbersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return PortingPhoneNumbersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> PortingPhoneNumbersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return PortingPhoneNumbersResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        filter: porting_phone_number_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[PortingPhoneNumberListResponse]:
        """
        Returns a list of your porting phone numbers.

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[porting_order_status]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/porting_phone_numbers",
            page=SyncDefaultFlatPagination[PortingPhoneNumberListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    porting_phone_number_list_params.PortingPhoneNumberListParams,
                ),
            ),
            model=PortingPhoneNumberListResponse,
        )


class AsyncPortingPhoneNumbersResource(AsyncAPIResource):
    """Endpoints related to porting orders management."""

    @cached_property
    def with_raw_response(self) -> AsyncPortingPhoneNumbersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncPortingPhoneNumbersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncPortingPhoneNumbersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncPortingPhoneNumbersResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        filter: porting_phone_number_list_params.Filter | Omit = omit,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[PortingPhoneNumberListResponse, AsyncDefaultFlatPagination[PortingPhoneNumberListResponse]]:
        """
        Returns a list of your porting phone numbers.

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[porting_order_status]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/porting_phone_numbers",
            page=AsyncDefaultFlatPagination[PortingPhoneNumberListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    porting_phone_number_list_params.PortingPhoneNumberListParams,
                ),
            ),
            model=PortingPhoneNumberListResponse,
        )


class PortingPhoneNumbersResourceWithRawResponse:
    def __init__(self, porting_phone_numbers: PortingPhoneNumbersResource) -> None:
        self._porting_phone_numbers = porting_phone_numbers

        self.list = to_raw_response_wrapper(
            porting_phone_numbers.list,
        )


class AsyncPortingPhoneNumbersResourceWithRawResponse:
    def __init__(self, porting_phone_numbers: AsyncPortingPhoneNumbersResource) -> None:
        self._porting_phone_numbers = porting_phone_numbers

        self.list = async_to_raw_response_wrapper(
            porting_phone_numbers.list,
        )


class PortingPhoneNumbersResourceWithStreamingResponse:
    def __init__(self, porting_phone_numbers: PortingPhoneNumbersResource) -> None:
        self._porting_phone_numbers = porting_phone_numbers

        self.list = to_streamed_response_wrapper(
            porting_phone_numbers.list,
        )


class AsyncPortingPhoneNumbersResourceWithStreamingResponse:
    def __init__(self, porting_phone_numbers: AsyncPortingPhoneNumbersResource) -> None:
        self._porting_phone_numbers = porting_phone_numbers

        self.list = async_to_streamed_response_wrapper(
            porting_phone_numbers.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ....._types import Body, Query, Headers, NoneType, NotGiven, not_given
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._base_client import make_request_options

__all__ = ["InsightsResource", "AsyncInsightsResource"]


class InsightsResource(SyncAPIResource):
    """Manage historical AI assistant conversations"""

    @cached_property
    def with_raw_response(self) -> InsightsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return InsightsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> InsightsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return InsightsResourceWithStreamingResponse(self)

    def assign(
        self,
        insight_id: str,
        *,
        group_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Assign an insight to a group

        Args:
          group_id: The ID of the insight group

          insight_id: The ID of the insight

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not group_id:
            raise ValueError(f"Expected a non-empty value for `group_id` but received {group_id!r}")
        if not insight_id:
            raise ValueError(f"Expected a non-empty value for `insight_id` but received {insight_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._post(
            f"/ai/conversations/insight-groups/{group_id}/insights/{insight_id}/assign",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def delete_unassign(
        self,
        insight_id: str,
        *,
        group_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Remove an insight from a group

        Args:
          group_id: The ID of the insight group

          insight_id: The ID of the insight

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not group_id:
            raise ValueError(f"Expected a non-empty value for `group_id` but received {group_id!r}")
        if not insight_id:
            raise ValueError(f"Expected a non-empty value for `insight_id` but received {insight_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/ai/conversations/insight-groups/{group_id}/insights/{insight_id}/unassign",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AsyncInsightsResource(AsyncAPIResource):
    """Manage historical AI assistant conversations"""

    @cached_property
    def with_raw_response(self) -> AsyncInsightsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncInsightsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncInsightsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncInsightsResourceWithStreamingResponse(self)

    async def assign(
        self,
        insight_id: str,
        *,
        group_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Assign an insight to a group

        Args:
          group_id: The ID of the insight group

          insight_id: The ID of the insight

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not group_id:
            raise ValueError(f"Expected a non-empty value for `group_id` but received {group_id!r}")
        if not insight_id:
            raise ValueError(f"Expected a non-empty value for `insight_id` but received {insight_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._post(
            f"/ai/conversations/insight-groups/{group_id}/insights/{insight_id}/assign",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def delete_unassign(
        self,
        insight_id: str,
        *,
        group_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Remove an insight from a group

        Args:
          group_id: The ID of the insight group

          insight_id: The ID of the insight

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not group_id:
            raise ValueError(f"Expected a non-empty value for `group_id` but received {group_id!r}")
        if not insight_id:
            raise ValueError(f"Expected a non-empty value for `insight_id` but received {insight_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/ai/conversations/insight-groups/{group_id}/insights/{insight_id}/unassign",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class InsightsResourceWithRawResponse:
    def __init__(self, insights: InsightsResource) -> None:
        self._insights = insights

        self.assign = to_raw_response_wrapper(
            insights.assign,
        )
        self.delete_unassign = to_raw_response_wrapper(
            insights.delete_unassign,
        )


class AsyncInsightsResourceWithRawResponse:
    def __init__(self, insights: AsyncInsightsResource) -> None:
        self._insights = insights

        self.assign = async_to_raw_response_wrapper(
            insights.assign,
        )
        self.delete_unassign = async_to_raw_response_wrapper(
            insights.delete_unassign,
        )


class InsightsResourceWithStreamingResponse:
    def __init__(self, insights: InsightsResource) -> None:
        self._insights = insights

        self.assign = to_streamed_response_wrapper(
            insights.assign,
        )
        self.delete_unassign = to_streamed_response_wrapper(
            insights.delete_unassign,
        )


class AsyncInsightsResourceWithStreamingResponse:
    def __init__(self, insights: AsyncInsightsResource) -> None:
        self._insights = insights

        self.assign = async_to_streamed_response_wrapper(
            insights.assign,
        )
        self.delete_unassign = async_to_streamed_response_wrapper(
            insights.delete_unassign,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["TextToSpeechListVoicesParams"]


class TextToSpeechListVoicesParams(TypedDict, total=False):
    api_key: str
    """API key for providers that require one to list voices (e.g. ElevenLabs)."""

    provider: Literal["aws", "telnyx", "azure", "elevenlabs", "minimax", "rime", "resemble", "inworld"]
    """Filter voices by provider. If omitted, voices from all providers are returned."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from pydantic import Field as FieldInfo

from .url import URL
from .volume import Volume
from ...._models import BaseModel
from .tf_phone_number import TfPhoneNumber
from .use_case_categories import UseCaseCategories
from .tf_verification_status import TfVerificationStatus
from .toll_free_verification_entity_type import TollFreeVerificationEntityType

__all__ = ["VerificationRequestEgress"]


class VerificationRequestEgress(BaseModel):
    """A verification request as it comes out of the database"""

    id: str

    additional_information: str = FieldInfo(alias="additionalInformation")

    business_addr1: str = FieldInfo(alias="businessAddr1")

    business_city: str = FieldInfo(alias="businessCity")

    business_contact_email: str = FieldInfo(alias="businessContactEmail")

    business_contact_first_name: str = FieldInfo(alias="businessContactFirstName")

    business_contact_last_name: str = FieldInfo(alias="businessContactLastName")

    business_contact_phone: str = FieldInfo(alias="businessContactPhone")

    business_name: str = FieldInfo(alias="businessName")

    business_state: str = FieldInfo(alias="businessState")

    business_zip: str = FieldInfo(alias="businessZip")

    corporate_website: str = FieldInfo(alias="corporateWebsite")

    message_volume: Volume = FieldInfo(alias="messageVolume")
    """Message Volume Enums"""

    opt_in_workflow: str = FieldInfo(alias="optInWorkflow")

    opt_in_workflow_image_urls: List[URL] = FieldInfo(alias="optInWorkflowImageURLs")

    phone_numbers: List[TfPhoneNumber] = FieldInfo(alias="phoneNumbers")

    production_message_content: str = FieldInfo(alias="productionMessageContent")

    use_case: UseCaseCategories = FieldInfo(alias="useCase")
    """Tollfree usecase categories"""

    use_case_summary: str = FieldInfo(alias="useCaseSummary")

    verification_request_id: str = FieldInfo(alias="verificationRequestId")

    age_gated_content: Optional[bool] = FieldInfo(alias="ageGatedContent", default=None)

    business_addr2: Optional[str] = FieldInfo(alias="businessAddr2", default=None)

    business_registration_country: Optional[str] = FieldInfo(alias="businessRegistrationCountry", default=None)

    business_registration_number: Optional[str] = FieldInfo(alias="businessRegistrationNumber", default=None)

    business_registration_type: Optional[str] = FieldInfo(alias="businessRegistrationType", default=None)

    campaign_verify_authorization_token: Optional[str] = FieldInfo(
        alias="campaignVerifyAuthorizationToken", default=None
    )
    """
    Campaign Verify Authorization Token required for Political use case submissions
    starting February 17, 2026
    """

    doing_business_as: Optional[str] = FieldInfo(alias="doingBusinessAs", default=None)

    entity_type: Optional[TollFreeVerificationEntityType] = FieldInfo(alias="entityType", default=None)
    """Business entity classification"""

    help_message_response: Optional[str] = FieldInfo(alias="helpMessageResponse", default=None)

    isv_reseller: Optional[str] = FieldInfo(alias="isvReseller", default=None)

    opt_in_confirmation_response: Optional[str] = FieldInfo(alias="optInConfirmationResponse", default=None)

    opt_in_keywords: Optional[str] = FieldInfo(alias="optInKeywords", default=None)

    privacy_policy_url: Optional[str] = FieldInfo(alias="privacyPolicyURL", default=None)

    terms_and_condition_url: Optional[str] = FieldInfo(alias="termsAndConditionURL", default=None)

    verification_status: Optional[TfVerificationStatus] = FieldInfo(alias="verificationStatus", default=None)
    """Tollfree verification status"""

    webhook_url: Optional[str] = FieldInfo(alias="webhookUrl", default=None)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel
from .csv_download import CsvDownload

__all__ = ["CsvDownloadRetrieveResponse"]


class CsvDownloadRetrieveResponse(BaseModel):
    data: Optional[List[CsvDownload]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .external_vetting_order_params import ExternalVettingOrderParams as ExternalVettingOrderParams
from .external_vetting_list_response import ExternalVettingListResponse as ExternalVettingListResponse
from .external_vetting_imports_params import ExternalVettingImportsParams as ExternalVettingImportsParams
from .external_vetting_order_response import ExternalVettingOrderResponse as ExternalVettingOrderResponse
from .external_vetting_imports_response import ExternalVettingImportsResponse as ExternalVettingImportsResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import date

from ..._models import BaseModel

__all__ = ["BillingBundleSummary"]


class BillingBundleSummary(BaseModel):
    id: str
    """Bundle's ID, this is used to identify the bundle in the API."""

    cost_code: str
    """Bundle's cost code, this is used to identify the bundle in the billing system."""

    created_at: date
    """Date the bundle was created."""

    is_public: bool
    """Available to all customers or only to specific customers."""

    name: str
    """Bundle's name, this is used to identify the bundle in the UI."""

    currency: Optional[str] = None
    """Bundle's currency code."""

    mrc_price: Optional[float] = None
    """Monthly recurring charge price."""

    slug: Optional[str] = None
    """Slugified version of the bundle's name."""

    specs: Optional[List[str]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from .calls import (
    CallsResource,
    AsyncCallsResource,
    CallsResourceWithRawResponse,
    AsyncCallsResourceWithRawResponse,
    CallsResourceWithStreamingResponse,
    AsyncCallsResourceWithStreamingResponse,
)
from ...types import queue_list_params, queue_create_params, queue_update_params
from ..._types import Body, Omit, Query, Headers, NoneType, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from ...types.queue import Queue
from ..._base_client import AsyncPaginator, make_request_options
from ...types.queue_create_response import QueueCreateResponse
from ...types.queue_update_response import QueueUpdateResponse
from ...types.queue_retrieve_response import QueueRetrieveResponse

__all__ = ["QueuesResource", "AsyncQueuesResource"]


class QueuesResource(SyncAPIResource):
    """Queue commands operations"""

    @cached_property
    def calls(self) -> CallsResource:
        """Queue commands operations"""
        return CallsResource(self._client)

    @cached_property
    def with_raw_response(self) -> QueuesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return QueuesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> QueuesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return QueuesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        queue_name: str,
        max_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> QueueCreateResponse:
        """Create a new call queue.

        Args:
          queue_name: The name of the queue.

        Must be between 1 and 255 characters.

          max_size: The maximum number of calls allowed in the queue.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/queues",
            body=maybe_transform(
                {
                    "queue_name": queue_name,
                    "max_size": max_size,
                },
                queue_create_params.QueueCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=QueueCreateResponse,
        )

    def retrieve(
        self,
        queue_name: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> QueueRetrieveResponse:
        """
        Retrieve an existing call queue

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not queue_name:
            raise ValueError(f"Expected a non-empty value for `queue_name` but received {queue_name!r}")
        return self._get(
            f"/queues/{queue_name}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=QueueRetrieveResponse,
        )

    def update(
        self,
        queue_name: str,
        *,
        max_size: int,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> QueueUpdateResponse:
        """
        Update properties of an existing call queue.

        Args:
          max_size: The maximum number of calls allowed in the queue.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not queue_name:
            raise ValueError(f"Expected a non-empty value for `queue_name` but received {queue_name!r}")
        return self._post(
            f"/queues/{queue_name}",
            body=maybe_transform({"max_size": max_size}, queue_update_params.QueueUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=QueueUpdateResponse,
        )

    def list(
        self,
        *,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[Queue]:
        """
        List all queues for the authenticated user.

        Args:
          page_number: The page number to load

          page_size: The size of the page

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/queues",
            page=SyncDefaultFlatPagination[Queue],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    queue_list_params.QueueListParams,
                ),
            ),
            model=Queue,
        )

    def delete(
        self,
        queue_name: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete an existing call queue.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not queue_name:
            raise ValueError(f"Expected a non-empty value for `queue_name` but received {queue_name!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/queues/{queue_name}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AsyncQueuesResource(AsyncAPIResource):
    """Queue commands operations"""

    @cached_property
    def calls(self) -> AsyncCallsResource:
        """Queue commands operations"""
        return AsyncCallsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncQueuesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncQueuesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncQueuesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncQueuesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        queue_name: str,
        max_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> QueueCreateResponse:
        """Create a new call queue.

        Args:
          queue_name: The name of the queue.

        Must be between 1 and 255 characters.

          max_size: The maximum number of calls allowed in the queue.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/queues",
            body=await async_maybe_transform(
                {
                    "queue_name": queue_name,
                    "max_size": max_size,
                },
                queue_create_params.QueueCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=QueueCreateResponse,
        )

    async def retrieve(
        self,
        queue_name: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> QueueRetrieveResponse:
        """
        Retrieve an existing call queue

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not queue_name:
            raise ValueError(f"Expected a non-empty value for `queue_name` but received {queue_name!r}")
        return await self._get(
            f"/queues/{queue_name}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=QueueRetrieveResponse,
        )

    async def update(
        self,
        queue_name: str,
        *,
        max_size: int,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> QueueUpdateResponse:
        """
        Update properties of an existing call queue.

        Args:
          max_size: The maximum number of calls allowed in the queue.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not queue_name:
            raise ValueError(f"Expected a non-empty value for `queue_name` but received {queue_name!r}")
        return await self._post(
            f"/queues/{queue_name}",
            body=await async_maybe_transform({"max_size": max_size}, queue_update_params.QueueUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=QueueUpdateResponse,
        )

    def list(
        self,
        *,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[Queue, AsyncDefaultFlatPagination[Queue]]:
        """
        List all queues for the authenticated user.

        Args:
          page_number: The page number to load

          page_size: The size of the page

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/queues",
            page=AsyncDefaultFlatPagination[Queue],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    queue_list_params.QueueListParams,
                ),
            ),
            model=Queue,
        )

    async def delete(
        self,
        queue_name: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete an existing call queue.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not queue_name:
            raise ValueError(f"Expected a non-empty value for `queue_name` but received {queue_name!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/queues/{queue_name}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class QueuesResourceWithRawResponse:
    def __init__(self, queues: QueuesResource) -> None:
        self._queues = queues

        self.create = to_raw_response_wrapper(
            queues.create,
        )
        self.retrieve = to_raw_response_wrapper(
            queues.retrieve,
        )
        self.update = to_raw_response_wrapper(
            queues.update,
        )
        self.list = to_raw_response_wrapper(
            queues.list,
        )
        self.delete = to_raw_response_wrapper(
            queues.delete,
        )

    @cached_property
    def calls(self) -> CallsResourceWithRawResponse:
        """Queue commands operations"""
        return CallsResourceWithRawResponse(self._queues.calls)


class AsyncQueuesResourceWithRawResponse:
    def __init__(self, queues: AsyncQueuesResource) -> None:
        self._queues = queues

        self.create = async_to_raw_response_wrapper(
            queues.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            queues.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            queues.update,
        )
        self.list = async_to_raw_response_wrapper(
            queues.list,
        )
        self.delete = async_to_raw_response_wrapper(
            queues.delete,
        )

    @cached_property
    def calls(self) -> AsyncCallsResourceWithRawResponse:
        """Queue commands operations"""
        return AsyncCallsResourceWithRawResponse(self._queues.calls)


class QueuesResourceWithStreamingResponse:
    def __init__(self, queues: QueuesResource) -> None:
        self._queues = queues

        self.create = to_streamed_response_wrapper(
            queues.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            queues.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            queues.update,
        )
        self.list = to_streamed_response_wrapper(
            queues.list,
        )
        self.delete = to_streamed_response_wrapper(
            queues.delete,
        )

    @cached_property
    def calls(self) -> CallsResourceWithStreamingResponse:
        """Queue commands operations"""
        return CallsResourceWithStreamingResponse(self._queues.calls)


class AsyncQueuesResourceWithStreamingResponse:
    def __init__(self, queues: AsyncQueuesResource) -> None:
        self._queues = queues

        self.create = async_to_streamed_response_wrapper(
            queues.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            queues.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            queues.update,
        )
        self.list = async_to_streamed_response_wrapper(
            queues.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            queues.delete,
        )

    @cached_property
    def calls(self) -> AsyncCallsResourceWithStreamingResponse:
        """Queue commands operations"""
        return AsyncCallsResourceWithStreamingResponse(self._queues.calls)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .ip import IP
from .._models import BaseModel

__all__ = ["IPCreateResponse"]


class IPCreateResponse(BaseModel):
    data: Optional[IP] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from ...._models import BaseModel

__all__ = ["PhoneNumberListResponse"]


class PhoneNumberListResponse(BaseModel):
    calling_enabled: Optional[bool] = None

    created_at: Optional[datetime] = None

    display_name: Optional[str] = None

    enabled: Optional[bool] = None

    phone_number: Optional[str] = None
    """Phone number in E164 format"""

    phone_number_id: Optional[str] = None
    """Whatsapp phone number ID"""

    quality_rating: Optional[str] = None
    """Whatsapp quality rating"""

    record_type: Optional[str] = None

    status: Optional[str] = None

    user_id: Optional[str] = None
    """User ID"""

    waba_id: Optional[str] = None
    """WABA ID of Whatsapp business account"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Literal, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["SubNumberOrdersReportCreateParams"]


class SubNumberOrdersReportCreateParams(TypedDict, total=False):
    country_code: str
    """Filter by country code"""

    created_at_gt: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Filter for orders created after this date"""

    created_at_lt: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Filter for orders created before this date"""

    customer_reference: str
    """Filter by customer reference"""

    order_request_id: str
    """Filter by specific order request ID"""

    status: Literal["pending", "success", "failure"]
    """Filter by order status"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from .._models import BaseModel

__all__ = ["NumberOrderPhoneNumberUpdateRequirementGroupResponse", "Data", "DataRegulatoryRequirement"]


class DataRegulatoryRequirement(BaseModel):
    field_type: Optional[str] = None

    field_value: Optional[str] = None

    requirement_id: Optional[str] = None

    status: Optional[str] = None


class Data(BaseModel):
    id: Optional[str] = None

    bundle_id: Optional[str] = None

    country_code: Optional[str] = None

    deadline: Optional[datetime] = None

    is_block_number: Optional[bool] = None

    locality: Optional[str] = None

    order_request_id: Optional[str] = None

    phone_number: Optional[str] = None

    phone_number_type: Optional[str] = None

    record_type: Optional[str] = None

    regulatory_requirements: Optional[List[DataRegulatoryRequirement]] = None

    requirements_met: Optional[bool] = None

    requirements_status: Optional[str] = None

    status: Optional[str] = None

    sub_number_order_id: Optional[str] = None


class NumberOrderPhoneNumberUpdateRequirementGroupResponse(BaseModel):
    data: Optional[Data] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Required, TypedDict

__all__ = ["VerificationTriggerCallParams"]


class VerificationTriggerCallParams(TypedDict, total=False):
    phone_number: Required[str]
    """+E164 formatted phone number."""

    verify_profile_id: Required[str]
    """The identifier of the associated Verify profile."""

    custom_code: Optional[str]
    """Send a self-generated numeric code to the end-user"""

    extension: Optional[str]
    """Optional extension to dial after call is answered using DTMF digits.

    Valid digits are 0-9, A-D, \\**, and #. Pauses can be added using w (0.5s) and W
    (1s).
    """

    timeout_secs: int
    """The number of seconds the verification code is valid for."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ....._models import BaseModel

__all__ = ["TelcoDataAggregation"]


class TelcoDataAggregation(BaseModel):
    currency: Optional[str] = None
    """Currency code"""

    total_cost: Optional[float] = None
    """Total cost for this aggregation"""

    total_dips: Optional[int] = None
    """Total number of lookups performed"""

    type: Optional[str] = None
    """Type of telco data lookup"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from telnyx.types.sim_card_groups import (
    SimCardGroupAction,
    ActionRetrieveResponse,
    ActionSetWirelessBlocklistResponse,
    ActionRemoveWirelessBlocklistResponse,
    ActionSetPrivateWirelessGatewayResponse,
    ActionRemovePrivateWirelessGatewayResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestActions:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Telnyx) -> None:
        action = client.sim_card_groups.actions.retrieve(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )
        assert_matches_type(ActionRetrieveResponse, action, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Telnyx) -> None:
        response = client.sim_card_groups.actions.with_raw_response.retrieve(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        action = response.parse()
        assert_matches_type(ActionRetrieveResponse, action, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Telnyx) -> None:
        with client.sim_card_groups.actions.with_streaming_response.retrieve(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            action = response.parse()
            assert_matches_type(ActionRetrieveResponse, action, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.sim_card_groups.actions.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list(self, client: Telnyx) -> None:
        action = client.sim_card_groups.actions.list()
        assert_matches_type(SyncDefaultFlatPagination[SimCardGroupAction], action, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Telnyx) -> None:
        action = client.sim_card_groups.actions.list(
            filter_sim_card_group_id="47a1c2b0-cc7b-4ab1-bb98-b33fb0fc61b9",
            filter_status="in-progress",
            filter_type="set_private_wireless_gateway",
            page_number=1,
            page_size=1,
        )
        assert_matches_type(SyncDefaultFlatPagination[SimCardGroupAction], action, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Telnyx) -> None:
        response = client.sim_card_groups.actions.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        action = response.parse()
        assert_matches_type(SyncDefaultFlatPagination[SimCardGroupAction], action, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Telnyx) -> None:
        with client.sim_card_groups.actions.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            action = response.parse()
            assert_matches_type(SyncDefaultFlatPagination[SimCardGroupAction], action, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_remove_private_wireless_gateway(self, client: Telnyx) -> None:
        action = client.sim_card_groups.actions.remove_private_wireless_gateway(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )
        assert_matches_type(ActionRemovePrivateWirelessGatewayResponse, action, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_remove_private_wireless_gateway(self, client: Telnyx) -> None:
        response = client.sim_card_groups.actions.with_raw_response.remove_private_wireless_gateway(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        action = response.parse()
        assert_matches_type(ActionRemovePrivateWirelessGatewayResponse, action, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_remove_private_wireless_gateway(self, client: Telnyx) -> None:
        with client.sim_card_groups.actions.with_streaming_response.remove_private_wireless_gateway(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            action = response.parse()
            assert_matches_type(ActionRemovePrivateWirelessGatewayResponse, action, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_remove_private_wireless_gateway(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.sim_card_groups.actions.with_raw_response.remove_private_wireless_gateway(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_remove_wireless_blocklist(self, client: Telnyx) -> None:
        action = client.sim_card_groups.actions.remove_wireless_blocklist(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )
        assert_matches_type(ActionRemoveWirelessBlocklistResponse, action, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_remove_wireless_blocklist(self, client: Telnyx) -> None:
        response = client.sim_card_groups.actions.with_raw_response.remove_wireless_blocklist(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        action = response.parse()
        assert_matches_type(ActionRemoveWirelessBlocklistResponse, action, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_remove_wireless_blocklist(self, client: Telnyx) -> None:
        with client.sim_card_groups.actions.with_streaming_response.remove_wireless_blocklist(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            action = response.parse()
            assert_matches_type(ActionRemoveWirelessBlocklistResponse, action, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_remove_wireless_blocklist(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.sim_card_groups.actions.with_raw_response.remove_wireless_blocklist(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_set_private_wireless_gateway(self, client: Telnyx) -> None:
        action = client.sim_card_groups.actions.set_private_wireless_gateway(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            private_wireless_gateway_id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )
        assert_matches_type(ActionSetPrivateWirelessGatewayResponse, action, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_set_private_wireless_gateway(self, client: Telnyx) -> None:
        response = client.sim_card_groups.actions.with_raw_response.set_private_wireless_gateway(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            private_wireless_gateway_id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        action = response.parse()
        assert_matches_type(ActionSetPrivateWirelessGatewayResponse, action, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_set_private_wireless_gateway(self, client: Telnyx) -> None:
        with client.sim_card_groups.actions.with_streaming_response.set_private_wireless_gateway(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            private_wireless_gateway_id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            action = response.parse()
            assert_matches_type(ActionSetPrivateWirelessGatewayResponse, action, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_set_private_wireless_gateway(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.sim_card_groups.actions.with_raw_response.set_private_wireless_gateway(
                id="",
                private_wireless_gateway_id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_set_wireless_blocklist(self, client: Telnyx) -> None:
        action = client.sim_card_groups.actions.set_wireless_blocklist(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            wireless_blocklist_id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )
        assert_matches_type(ActionSetWirelessBlocklistResponse, action, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_set_wireless_blocklist(self, client: Telnyx) -> None:
        response = client.sim_card_groups.actions.with_raw_response.set_wireless_blocklist(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            wireless_blocklist_id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        action = response.parse()
        assert_matches_type(ActionSetWirelessBlocklistResponse, action, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_set_wireless_blocklist(self, client: Telnyx) -> None:
        with client.sim_card_groups.actions.with_streaming_response.set_wireless_blocklist(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            wireless_blocklist_id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            action = response.parse()
            assert_matches_type(ActionSetWirelessBlocklistResponse, action, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_set_wireless_blocklist(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.sim_card_groups.actions.with_raw_response.set_wireless_blocklist(
                id="",
                wireless_blocklist_id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            )


class TestAsyncActions:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncTelnyx) -> None:
        action = await async_client.sim_card_groups.actions.retrieve(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )
        assert_matches_type(ActionRetrieveResponse, action, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.sim_card_groups.actions.with_raw_response.retrieve(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        action = await response.parse()
        assert_matches_type(ActionRetrieveResponse, action, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        async with async_client.sim_card_groups.actions.with_streaming_response.retrieve(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            action = await response.parse()
            assert_matches_type(ActionRetrieveResponse, action, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.sim_card_groups.actions.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncTelnyx) -> None:
        action = await async_client.sim_card_groups.actions.list()
        assert_matches_type(AsyncDefaultFlatPagination[SimCardGroupAction], action, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncTelnyx) -> None:
        action = await async_client.sim_card_groups.actions.list(
            filter_sim_card_group_id="47a1c2b0-cc7b-4ab1-bb98-b33fb0fc61b9",
            filter_status="in-progress",
            filter_type="set_private_wireless_gateway",
            page_number=1,
            page_size=1,
        )
        assert_matches_type(AsyncDefaultFlatPagination[SimCardGroupAction], action, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.sim_card_groups.actions.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        action = await response.parse()
        assert_matches_type(AsyncDefaultFlatPagination[SimCardGroupAction], action, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncTelnyx) -> None:
        async with async_client.sim_card_groups.actions.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            action = await response.parse()
            assert_matches_type(AsyncDefaultFlatPagination[SimCardGroupAction], action, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_remove_private_wireless_gateway(self, async_client: AsyncTelnyx) -> None:
        action = await async_client.sim_card_groups.actions.remove_private_wireless_gateway(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )
        assert_matches_type(ActionRemovePrivateWirelessGatewayResponse, action, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_remove_private_wireless_gateway(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.sim_card_groups.actions.with_raw_response.remove_private_wireless_gateway(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        action = await response.parse()
        assert_matches_type(ActionRemovePrivateWirelessGatewayResponse, action, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_remove_private_wireless_gateway(self, async_client: AsyncTelnyx) -> None:
        async with async_client.sim_card_groups.actions.with_streaming_response.remove_private_wireless_gateway(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            action = await response.parse()
            assert_matches_type(ActionRemovePrivateWirelessGatewayResponse, action, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_remove_private_wireless_gateway(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.sim_card_groups.actions.with_raw_response.remove_private_wireless_gateway(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_remove_wireless_blocklist(self, async_client: AsyncTelnyx) -> None:
        action = await async_client.sim_card_groups.actions.remove_wireless_blocklist(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )
        assert_matches_type(ActionRemoveWirelessBlocklistResponse, action, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_remove_wireless_blocklist(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.sim_card_groups.actions.with_raw_response.remove_wireless_blocklist(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        action = await response.parse()
        assert_matches_type(ActionRemoveWirelessBlocklistResponse, action, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_remove_wireless_blocklist(self, async_client: AsyncTelnyx) -> None:
        async with async_client.sim_card_groups.actions.with_streaming_response.remove_wireless_blocklist(
            "6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            action = await response.parse()
            assert_matches_type(ActionRemoveWirelessBlocklistResponse, action, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_remove_wireless_blocklist(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.sim_card_groups.actions.with_raw_response.remove_wireless_blocklist(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_set_private_wireless_gateway(self, async_client: AsyncTelnyx) -> None:
        action = await async_client.sim_card_groups.actions.set_private_wireless_gateway(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            private_wireless_gateway_id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )
        assert_matches_type(ActionSetPrivateWirelessGatewayResponse, action, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_set_private_wireless_gateway(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.sim_card_groups.actions.with_raw_response.set_private_wireless_gateway(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            private_wireless_gateway_id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        action = await response.parse()
        assert_matches_type(ActionSetPrivateWirelessGatewayResponse, action, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_set_private_wireless_gateway(self, async_client: AsyncTelnyx) -> None:
        async with async_client.sim_card_groups.actions.with_streaming_response.set_private_wireless_gateway(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            private_wireless_gateway_id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            action = await response.parse()
            assert_matches_type(ActionSetPrivateWirelessGatewayResponse, action, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_set_private_wireless_gateway(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.sim_card_groups.actions.with_raw_response.set_private_wireless_gateway(
                id="",
                private_wireless_gateway_id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_set_wireless_blocklist(self, async_client: AsyncTelnyx) -> None:
        action = await async_client.sim_card_groups.actions.set_wireless_blocklist(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            wireless_blocklist_id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )
        assert_matches_type(ActionSetWirelessBlocklistResponse, action, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_set_wireless_blocklist(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.sim_card_groups.actions.with_raw_response.set_wireless_blocklist(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            wireless_blocklist_id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        action = await response.parse()
        assert_matches_type(ActionSetWirelessBlocklistResponse, action, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_set_wireless_blocklist(self, async_client: AsyncTelnyx) -> None:
        async with async_client.sim_card_groups.actions.with_streaming_response.set_wireless_blocklist(
            id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            wireless_blocklist_id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            action = await response.parse()
            assert_matches_type(ActionSetWirelessBlocklistResponse, action, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_set_wireless_blocklist(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.sim_card_groups.actions.with_raw_response.set_wireless_blocklist(
                id="",
                wireless_blocklist_id="6a09cdc3-8948-47f0-aa62-74ac943d6c58",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import TypeAlias, TypedDict

__all__ = [
    "GlobalIPAssignmentsUsageRetrieveParams",
    "Filter",
    "FilterGlobalIPAssignmentID",
    "FilterGlobalIPAssignmentIDIn",
    "FilterGlobalIPID",
    "FilterGlobalIPIDIn",
]


class GlobalIPAssignmentsUsageRetrieveParams(TypedDict, total=False):
    filter: Filter
    """Consolidated filter parameter (deepObject style).

    Originally: filter[global_ip_assignment_id][in], filter[global_ip_id][in]
    """


_FilterGlobalIPAssignmentIDInReservedKeywords = TypedDict(
    "_FilterGlobalIPAssignmentIDInReservedKeywords",
    {
        "in": str,
    },
    total=False,
)


class FilterGlobalIPAssignmentIDIn(_FilterGlobalIPAssignmentIDInReservedKeywords, total=False):
    """Filtering operations"""

    pass


FilterGlobalIPAssignmentID: TypeAlias = Union[str, FilterGlobalIPAssignmentIDIn]

_FilterGlobalIpidInReservedKeywords = TypedDict(
    "_FilterGlobalIpidInReservedKeywords",
    {
        "in": str,
    },
    total=False,
)


class FilterGlobalIPIDIn(_FilterGlobalIpidInReservedKeywords, total=False):
    """Filtering operations"""

    pass


FilterGlobalIPID: TypeAlias = Union[str, FilterGlobalIPIDIn]


class Filter(TypedDict, total=False):
    """Consolidated filter parameter (deepObject style).

    Originally: filter[global_ip_assignment_id][in], filter[global_ip_id][in]
    """

    global_ip_assignment_id: FilterGlobalIPAssignmentID
    """Filter by exact Global IP Assignment ID"""

    global_ip_id: FilterGlobalIPID
    """Filter by exact Global IP ID"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.porting_orders import action_share_params
from ...types.porting_orders.action_share_response import ActionShareResponse
from ...types.porting_orders.action_cancel_response import ActionCancelResponse
from ...types.porting_orders.action_confirm_response import ActionConfirmResponse
from ...types.porting_orders.action_activate_response import ActionActivateResponse

__all__ = ["ActionsResource", "AsyncActionsResource"]


class ActionsResource(SyncAPIResource):
    """Endpoints related to porting orders management."""

    @cached_property
    def with_raw_response(self) -> ActionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return ActionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ActionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return ActionsResourceWithStreamingResponse(self)

    def activate(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionActivateResponse:
        """Activate each number in a porting order asynchronously.

        This operation is
        limited to US FastPort orders only.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/porting_orders/{id}/actions/activate",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionActivateResponse,
        )

    def cancel(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionCancelResponse:
        """
        Cancel a porting order

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/porting_orders/{id}/actions/cancel",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionCancelResponse,
        )

    def confirm(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionConfirmResponse:
        """
        Confirm and submit your porting order.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/porting_orders/{id}/actions/confirm",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionConfirmResponse,
        )

    def share(
        self,
        id: str,
        *,
        expires_in_seconds: int | Omit = omit,
        permissions: Literal["porting_order.document.read", "porting_order.document.update"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionShareResponse:
        """Creates a sharing token for a porting order.

        The token can be used to share the
        porting order with non-Telnyx users.

        Args:
          expires_in_seconds: The number of seconds the token will be valid for

          permissions: The permissions the token will have

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/porting_orders/{id}/actions/share",
            body=maybe_transform(
                {
                    "expires_in_seconds": expires_in_seconds,
                    "permissions": permissions,
                },
                action_share_params.ActionShareParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionShareResponse,
        )


class AsyncActionsResource(AsyncAPIResource):
    """Endpoints related to porting orders management."""

    @cached_property
    def with_raw_response(self) -> AsyncActionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncActionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncActionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncActionsResourceWithStreamingResponse(self)

    async def activate(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionActivateResponse:
        """Activate each number in a porting order asynchronously.

        This operation is
        limited to US FastPort orders only.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/porting_orders/{id}/actions/activate",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionActivateResponse,
        )

    async def cancel(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionCancelResponse:
        """
        Cancel a porting order

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/porting_orders/{id}/actions/cancel",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionCancelResponse,
        )

    async def confirm(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionConfirmResponse:
        """
        Confirm and submit your porting order.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/porting_orders/{id}/actions/confirm",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionConfirmResponse,
        )

    async def share(
        self,
        id: str,
        *,
        expires_in_seconds: int | Omit = omit,
        permissions: Literal["porting_order.document.read", "porting_order.document.update"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ActionShareResponse:
        """Creates a sharing token for a porting order.

        The token can be used to share the
        porting order with non-Telnyx users.

        Args:
          expires_in_seconds: The number of seconds the token will be valid for

          permissions: The permissions the token will have

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/porting_orders/{id}/actions/share",
            body=await async_maybe_transform(
                {
                    "expires_in_seconds": expires_in_seconds,
                    "permissions": permissions,
                },
                action_share_params.ActionShareParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ActionShareResponse,
        )


class ActionsResourceWithRawResponse:
    def __init__(self, actions: ActionsResource) -> None:
        self._actions = actions

        self.activate = to_raw_response_wrapper(
            actions.activate,
        )
        self.cancel = to_raw_response_wrapper(
            actions.cancel,
        )
        self.confirm = to_raw_response_wrapper(
            actions.confirm,
        )
        self.share = to_raw_response_wrapper(
            actions.share,
        )


class AsyncActionsResourceWithRawResponse:
    def __init__(self, actions: AsyncActionsResource) -> None:
        self._actions = actions

        self.activate = async_to_raw_response_wrapper(
            actions.activate,
        )
        self.cancel = async_to_raw_response_wrapper(
            actions.cancel,
        )
        self.confirm = async_to_raw_response_wrapper(
            actions.confirm,
        )
        self.share = async_to_raw_response_wrapper(
            actions.share,
        )


class ActionsResourceWithStreamingResponse:
    def __init__(self, actions: ActionsResource) -> None:
        self._actions = actions

        self.activate = to_streamed_response_wrapper(
            actions.activate,
        )
        self.cancel = to_streamed_response_wrapper(
            actions.cancel,
        )
        self.confirm = to_streamed_response_wrapper(
            actions.confirm,
        )
        self.share = to_streamed_response_wrapper(
            actions.share,
        )


class AsyncActionsResourceWithStreamingResponse:
    def __init__(self, actions: AsyncActionsResource) -> None:
        self._actions = actions

        self.activate = async_to_streamed_response_wrapper(
            actions.activate,
        )
        self.cancel = async_to_streamed_response_wrapper(
            actions.cancel,
        )
        self.confirm = async_to_streamed_response_wrapper(
            actions.confirm,
        )
        self.share = async_to_streamed_response_wrapper(
            actions.share,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ..._models import BaseModel

__all__ = ["RcGenerateDeeplinkResponse", "Data"]


class Data(BaseModel):
    url: str
    """The generated deeplink URL"""


class RcGenerateDeeplinkResponse(BaseModel):
    data: Data



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.messages import rc_send_params, rc_generate_deeplink_params
from ...types.rcs_agent_message_param import RcsAgentMessageParam
from ...types.messages.rc_send_response import RcSendResponse
from ...types.messages.rc_generate_deeplink_response import RcGenerateDeeplinkResponse

__all__ = ["RcsResource", "AsyncRcsResource"]


class RcsResource(SyncAPIResource):
    """Send RCS messages"""

    @cached_property
    def with_raw_response(self) -> RcsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return RcsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> RcsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return RcsResourceWithStreamingResponse(self)

    def generate_deeplink(
        self,
        agent_id: str,
        *,
        body: str | Omit = omit,
        phone_number: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RcGenerateDeeplinkResponse:
        """
        Generate a deeplink URL that can be used to start an RCS conversation with a
        specific agent.

        Args:
          body: Pre-filled message body (URL encoded)

          phone_number: Phone number in E164 format (URL encoded)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        return self._get(
            f"/messages/rcs/deeplinks/{agent_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "body": body,
                        "phone_number": phone_number,
                    },
                    rc_generate_deeplink_params.RcGenerateDeeplinkParams,
                ),
            ),
            cast_to=RcGenerateDeeplinkResponse,
        )

    def send(
        self,
        *,
        agent_id: str,
        agent_message: RcsAgentMessageParam,
        messaging_profile_id: str,
        to: str,
        mms_fallback: rc_send_params.MmsFallback | Omit = omit,
        sms_fallback: rc_send_params.SMSFallback | Omit = omit,
        type: Literal["RCS"] | Omit = omit,
        webhook_url: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RcSendResponse:
        """
        Send an RCS message

        Args:
          agent_id: RCS Agent ID

          messaging_profile_id: A valid messaging profile ID

          to: Phone number in +E.164 format

          type: Message type - must be set to "RCS"

          webhook_url: The URL where webhooks related to this message will be sent.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/messages/rcs",
            body=maybe_transform(
                {
                    "agent_id": agent_id,
                    "agent_message": agent_message,
                    "messaging_profile_id": messaging_profile_id,
                    "to": to,
                    "mms_fallback": mms_fallback,
                    "sms_fallback": sms_fallback,
                    "type": type,
                    "webhook_url": webhook_url,
                },
                rc_send_params.RcSendParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RcSendResponse,
        )


class AsyncRcsResource(AsyncAPIResource):
    """Send RCS messages"""

    @cached_property
    def with_raw_response(self) -> AsyncRcsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncRcsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncRcsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncRcsResourceWithStreamingResponse(self)

    async def generate_deeplink(
        self,
        agent_id: str,
        *,
        body: str | Omit = omit,
        phone_number: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RcGenerateDeeplinkResponse:
        """
        Generate a deeplink URL that can be used to start an RCS conversation with a
        specific agent.

        Args:
          body: Pre-filled message body (URL encoded)

          phone_number: Phone number in E164 format (URL encoded)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        return await self._get(
            f"/messages/rcs/deeplinks/{agent_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "body": body,
                        "phone_number": phone_number,
                    },
                    rc_generate_deeplink_params.RcGenerateDeeplinkParams,
                ),
            ),
            cast_to=RcGenerateDeeplinkResponse,
        )

    async def send(
        self,
        *,
        agent_id: str,
        agent_message: RcsAgentMessageParam,
        messaging_profile_id: str,
        to: str,
        mms_fallback: rc_send_params.MmsFallback | Omit = omit,
        sms_fallback: rc_send_params.SMSFallback | Omit = omit,
        type: Literal["RCS"] | Omit = omit,
        webhook_url: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RcSendResponse:
        """
        Send an RCS message

        Args:
          agent_id: RCS Agent ID

          messaging_profile_id: A valid messaging profile ID

          to: Phone number in +E.164 format

          type: Message type - must be set to "RCS"

          webhook_url: The URL where webhooks related to this message will be sent.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/messages/rcs",
            body=await async_maybe_transform(
                {
                    "agent_id": agent_id,
                    "agent_message": agent_message,
                    "messaging_profile_id": messaging_profile_id,
                    "to": to,
                    "mms_fallback": mms_fallback,
                    "sms_fallback": sms_fallback,
                    "type": type,
                    "webhook_url": webhook_url,
                },
                rc_send_params.RcSendParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RcSendResponse,
        )


class RcsResourceWithRawResponse:
    def __init__(self, rcs: RcsResource) -> None:
        self._rcs = rcs

        self.generate_deeplink = to_raw_response_wrapper(
            rcs.generate_deeplink,
        )
        self.send = to_raw_response_wrapper(
            rcs.send,
        )


class AsyncRcsResourceWithRawResponse:
    def __init__(self, rcs: AsyncRcsResource) -> None:
        self._rcs = rcs

        self.generate_deeplink = async_to_raw_response_wrapper(
            rcs.generate_deeplink,
        )
        self.send = async_to_raw_response_wrapper(
            rcs.send,
        )


class RcsResourceWithStreamingResponse:
    def __init__(self, rcs: RcsResource) -> None:
        self._rcs = rcs

        self.generate_deeplink = to_streamed_response_wrapper(
            rcs.generate_deeplink,
        )
        self.send = to_streamed_response_wrapper(
            rcs.send,
        )


class AsyncRcsResourceWithStreamingResponse:
    def __init__(self, rcs: AsyncRcsResource) -> None:
        self._rcs = rcs

        self.generate_deeplink = async_to_streamed_response_wrapper(
            rcs.generate_deeplink,
        )
        self.send = async_to_streamed_response_wrapper(
            rcs.send,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .conference_participant import ConferenceParticipant

__all__ = ["ConferenceRetrieveParticipantResponse"]


class ConferenceRetrieveParticipantResponse(BaseModel):
    data: Optional[ConferenceParticipant] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .call_conversation_insights_generated import CallConversationInsightsGenerated

__all__ = ["CallConversationInsightsGeneratedWebhookEvent"]


class CallConversationInsightsGeneratedWebhookEvent(BaseModel):
    data: Optional[CallConversationInsightsGenerated] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .profile import (
    ProfileResource,
    AsyncProfileResource,
    ProfileResourceWithRawResponse,
    AsyncProfileResourceWithRawResponse,
    ProfileResourceWithStreamingResponse,
    AsyncProfileResourceWithStreamingResponse,
)
from .phone_numbers import (
    PhoneNumbersResource,
    AsyncPhoneNumbersResource,
    PhoneNumbersResourceWithRawResponse,
    AsyncPhoneNumbersResourceWithRawResponse,
    PhoneNumbersResourceWithStreamingResponse,
    AsyncPhoneNumbersResourceWithStreamingResponse,
)
from .calling_settings import (
    CallingSettingsResource,
    AsyncCallingSettingsResource,
    CallingSettingsResourceWithRawResponse,
    AsyncCallingSettingsResourceWithRawResponse,
    CallingSettingsResourceWithStreamingResponse,
    AsyncCallingSettingsResourceWithStreamingResponse,
)

__all__ = [
    "CallingSettingsResource",
    "AsyncCallingSettingsResource",
    "CallingSettingsResourceWithRawResponse",
    "AsyncCallingSettingsResourceWithRawResponse",
    "CallingSettingsResourceWithStreamingResponse",
    "AsyncCallingSettingsResourceWithStreamingResponse",
    "ProfileResource",
    "AsyncProfileResource",
    "ProfileResourceWithRawResponse",
    "AsyncProfileResourceWithRawResponse",
    "ProfileResourceWithStreamingResponse",
    "AsyncProfileResourceWithStreamingResponse",
    "PhoneNumbersResource",
    "AsyncPhoneNumbersResource",
    "PhoneNumbersResourceWithRawResponse",
    "AsyncPhoneNumbersResourceWithRawResponse",
    "PhoneNumbersResourceWithStreamingResponse",
    "AsyncPhoneNumbersResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import date
from typing_extensions import Literal, Annotated, TypedDict

from ....._types import SequenceNotStr
from ....._utils import PropertyInfo

__all__ = ["NumberLookupCreateParams"]


class NumberLookupCreateParams(TypedDict, total=False):
    aggregation_type: Annotated[Literal["ALL", "BY_ORGANIZATION_MEMBER"], PropertyInfo(alias="aggregationType")]
    """Type of aggregation for the report"""

    end_date: Annotated[Union[str, date], PropertyInfo(alias="endDate", format="iso8601")]
    """End date for the usage report"""

    managed_accounts: Annotated[SequenceNotStr[str], PropertyInfo(alias="managedAccounts")]
    """List of managed accounts to include in the report"""

    start_date: Annotated[Union[str, date], PropertyInfo(alias="startDate", format="iso8601")]
    """Start date for the usage report"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from ...._models import BaseModel

__all__ = ["ProfileRetrieveResponse", "Data"]


class Data(BaseModel):
    id: Optional[str] = None

    about: Optional[str] = None

    address: Optional[str] = None

    category: Optional[str] = None

    created_at: Optional[datetime] = None

    description: Optional[str] = None

    display_name: Optional[str] = None

    email: Optional[str] = None

    phone_number_id: Optional[str] = None
    """Whatsapp phone number ID"""

    profile_photo_url: Optional[str] = None

    record_type: Optional[str] = None

    updated_at: Optional[datetime] = None

    website: Optional[str] = None


class ProfileRetrieveResponse(BaseModel):
    data: Optional[Data] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Iterable
from typing_extensions import Literal, Required, TypeAlias, TypedDict

from ..._types import SequenceNotStr
from .bucket_ids_param import BucketIDsParam

__all__ = [
    "ChatCreateCompletionParams",
    "Message",
    "MessageContentTextAndImageArray",
    "ResponseFormat",
    "Tool",
    "ToolFunction",
    "ToolFunctionFunction",
    "ToolRetrieval",
]


class ChatCreateCompletionParams(TypedDict, total=False):
    messages: Required[Iterable[Message]]
    """A list of the previous chat messages for context."""

    api_key_ref: str
    """
    If you are using an external inference provider like xAI or OpenAI, this field
    allows you to pass along a reference to your API key. After creating an
    [integration secret](https://developers.telnyx.com/api-reference/integration-secrets/create-a-secret)
    for you API key, pass the secret's `identifier` in this field.
    """

    best_of: int
    """
    This is used with `use_beam_search` to determine how many candidate beams to
    explore.
    """

    early_stopping: bool
    """This is used with `use_beam_search`.

    If `true`, generation stops as soon as there are `best_of` complete candidates;
    if `false`, a heuristic is applied and the generation stops when is it very
    unlikely to find better candidates.
    """

    enable_thinking: bool
    """
    Whether to enable the thinking/reasoning phase for models that support it (e.g.,
    QwQ, Qwen3). When set to false, the model will skip the internal reasoning step
    and respond directly, which can reduce latency. Defaults to true.
    """

    frequency_penalty: float
    """Higher values will penalize the model from repeating the same output tokens."""

    guided_choice: SequenceNotStr[str]
    """If specified, the output will be exactly one of the choices."""

    guided_json: Dict[str, object]
    """Must be a valid JSON schema.

    If specified, the output will follow the JSON schema.
    """

    guided_regex: str
    """If specified, the output will follow the regex pattern."""

    length_penalty: float
    """This is used with `use_beam_search` to prefer shorter or longer completions."""

    logprobs: bool
    """Whether to return log probabilities of the output tokens or not.

    If true, returns the log probabilities of each output token returned in the
    `content` of `message`.
    """

    max_tokens: int
    """Maximum number of completion tokens the model should generate."""

    min_p: float
    """
    This is an alternative to `top_p` that
    [many prefer](https://github.com/huggingface/transformers/issues/27670). Must be
    in [0, 1].
    """

    model: str
    """The language model to chat with."""

    n: float
    """This will return multiple choices for you instead of a single chat completion."""

    presence_penalty: float
    """Higher values will penalize the model from repeating the same output tokens."""

    response_format: ResponseFormat
    """Use this is you want to guarantee a JSON output without defining a schema.

    For control over the schema, use `guided_json`.
    """

    stream: bool
    """Whether or not to stream data-only server-sent events as they become available."""

    temperature: float
    """Adjusts the "creativity" of the model.

    Lower values make the model more deterministic and repetitive, while higher
    values make the model more random and creative.
    """

    tool_choice: Literal["none", "auto", "required"]

    tools: Iterable[Tool]
    """
    The `function` tool type follows the same schema as the
    [OpenAI Chat Completions API](https://platform.openai.com/docs/api-reference/chat).
    The `retrieval` tool type is unique to Telnyx. You may pass a list of
    [embedded storage buckets](https://developers.telnyx.com/api-reference/embeddings/embed-documents)
    for retrieval-augmented generation.
    """

    top_logprobs: int
    """This is used with `logprobs`.

    An integer between 0 and 20 specifying the number of most likely tokens to
    return at each token position, each with an associated log probability.
    """

    top_p: float
    """An alternative or complement to `temperature`.

    This adjusts how many of the top possibilities to consider.
    """

    use_beam_search: bool
    """
    Setting this to `true` will allow the model to
    [explore more completion options](https://huggingface.co/blog/how-to-generate#beam-search).
    This is not supported by OpenAI.
    """


class MessageContentTextAndImageArray(TypedDict, total=False):
    type: Required[Literal["text", "image_url"]]

    image_url: str

    text: str


class Message(TypedDict, total=False):
    content: Required[Union[str, Iterable[MessageContentTextAndImageArray]]]

    role: Required[Literal["system", "user", "assistant", "tool"]]


class ResponseFormat(TypedDict, total=False):
    """Use this is you want to guarantee a JSON output without defining a schema.

    For control over the schema, use `guided_json`.
    """

    type: Required[Literal["text", "json_object"]]


class ToolFunctionFunction(TypedDict, total=False):
    name: Required[str]

    description: str

    parameters: Dict[str, object]


class ToolFunction(TypedDict, total=False):
    function: Required[ToolFunctionFunction]

    type: Required[Literal["function"]]


class ToolRetrieval(TypedDict, total=False):
    retrieval: Required[BucketIDsParam]

    type: Required[Literal["retrieval"]]


Tool: TypeAlias = Union[ToolFunction, ToolRetrieval]
